@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: doc2gih.c,v 1.15 2005/06/03 05:11:55 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - doc2gih.c */
 
 /*[
@@ -56,8 +52,8 @@ static char *RCSid() { return RCSid("$Id: doc2gih.c,v 1.15 2005/06/03 05:11:55 s
 #include "stdfn.h"
 #include "doc2x.h"
 
-void convert __PROTO((FILE *, FILE *));
-void process_line __PROTO((char *, FILE *));
+void convert(FILE *, FILE *);
+void process_line(char *, FILE *);
 
 int
 main (int argc, char **argv)
@@ -119,11 +115,24 @@ process_line(char *line, FILE *b)
 	    break;		/* ignore */
 	}
     case '#':{			/* latex table entry */
+	    if (line[1] == 'b' && line[2] == ' ') {	/* bullet */
+		fputs(" * ", b);
+		fputs(line+2, b);
+	    }  else if (line[1] == '#') {
+		fputs("   ", b);
+		fputs(line+2, b);
+	    }
 	    break;		/* ignore */
 	}
     case '=':{			/* latex index entry */
 	    break;		/* ignore */
 	}
+    case 'F':{			/* latex embedded figure */
+	    break;		/* ignore */
+	}
+    case 'D':{			/* html embedded figure */
+	    break;		/* ignore */
+	}
     case '%':{			/* troff table entry */
 	    break;		/* ignore */
 	}
