@@ -1,8 +1,3 @@
-/*
- * $Id: doc2x.h,v 1.8 2004/04/13 17:23:31 broeker Exp $
- *
- */
-
 /* GNUPLOT - doc2x.h */
 
 /*[
@@ -58,14 +53,16 @@
 #endif
 
 #if defined(DOCS_TERMDOC_MAIN) || defined(DOCS_XREF_MAIN)
-extern char *termtext[];
+extern char *termtext_en[];
+extern char *termtext_ja[];
+extern char **termtext;
 #else
 
 /* a complete lie, but they dont need it ! */
 # define TERM_DRIVER_H
 # define TERM_HELP
 
-char *termtext[] = {
+char *termtext_en[] = {
 # ifdef ALL_TERM_DOC
 #  include "allterm.h"
 # else
@@ -73,6 +70,20 @@ char *termtext[] = {
 # endif
     NULL
 };
+
+char *termtext_ja[] = {
+# ifdef ALL_TERM_DOC
+#  undef PS_COMMON_PROLOG_INFO
+#  undef PS_COMMON_DOC1
+#  undef PS_COMMON_DOC2
+#  define JAPANESE_DOC
+#    include "allterm-ja.h"
+#  undef JAPANESE_DOC
+# endif
+    NULL
+};
+
+char **termtext = termtext_en;
 #endif /* !DOCS_TERMDOC_MAIN */
 
 /* From termdoc.c */
@@ -85,6 +96,6 @@ extern char termdoc_filename[];
 #ifndef DOCS_TERMDOC_MAIN
 extern
 #endif
-       char *get_line __PROTO((char *, int, FILE *));
+       char *get_line(char *, int, FILE *);
 
 #endif /* DOCS_DOC2X_H */
