@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: doc2html.c,v 1.6.4.1 2014/12/14 22:02:39 markisch Exp $"); }
-#endif
-
 /* GNUPLOT - doc2html.c */
 
 /*[
@@ -51,10 +47,11 @@ static char *RCSid() { return RCSid("$Id: doc2html.c,v 1.6.4.1 2014/12/14 22:02:
 # include "config.h"
 #endif
 
-#ifdef WXHELP
+#if defined(WXHELP) || defined(FPC)
 /* The Microsoft help compiler splits topics and creates the index automatically.
-   To create help files for wxWidgets we need to do this manually.
-   Note that this might overwrite the stub index file from CVS. */
+   To create help files for wxWidgets or using the Free Pascal tool "chmcmd",
+   we need to do this manually.
+   Note that this will overwrite the stub index file from the repository. */
 #define SPLIT_FILES
 #define CREATE_INDEX
 #endif
@@ -70,8 +67,8 @@ static TBOOLEAN debug = FALSE;
 static char path[PATH_MAX];
 static const char name[] = "wgnuplot";
 
-void convert __PROTO((FILE *, FILE *, FILE *, FILE *));
-void process_line __PROTO((char *, FILE *, FILE *, FILE *));
+void convert(FILE *, FILE *, FILE *, FILE *);
+void process_line(char *, FILE *, FILE *, FILE *);
 
 int
 main (int argc, char **argv)
@@ -146,7 +143,11 @@ header(FILE *a, char * title)
     fprintf(a, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
     fprintf(a, "<html>\n");
     fprintf(a, "<head>\n");
+#ifndef JAPANESE_DOC
     fprintf(a, "<meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\">\n");
+#else
+    fprintf(a, "<meta http-equiv=\"Content-type\" content=\"text/html; charset=shift-jis\">\n");
+#endif
     fprintf(a, "<title>%s</title>\n", title);
     fprintf(a, "</head>\n");
     fprintf(a, "<body>\n");
@@ -168,7 +169,7 @@ convert(FILE *a, FILE *b, FILE *c, FILE *d)
     static char line[MAX_LINE_LEN+1];
 
     header(b, "gnuplot help");
-    fprintf(b, "<h1 align=\"center\">gnuplot %s patchlevel %s</h1>\n", gnuplot_version, gnuplot_patchlevel);
+    fprintf(b, "<h1 align=\"center\">gnuplot %s patchlevel %s</h1>\n", VERSION_MAJOR, PATCHLEVEL);
 
     header(c, "gnuplot help contents");
     fprintf(c, "<ul>\n");
@@ -206,6 +207,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
     static int level = 1;
     static TBOOLEAN startpage = TRUE;
     static TBOOLEAN tabl = FALSE;
+    static TBOOLEAN intable = FALSE;
     static TBOOLEAN skiptable = FALSE;
     static TBOOLEAN forcetable = FALSE;
     static TBOOLEAN para = FALSE;
@@ -273,6 +275,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                     if (debug)
                         fprintf(stderr, "hyper link \"%s\" - %s on line %d\n", topic, id, line_count);
 #else
+		    (void)klink;	/* Otherwise compiler warning about unused variable */
                     /* explicit links */
                     if ((klist->line) > 1)
                         sprintf(hyplink1, "<a href=\"loc%d.html\">", klist->line);
@@ -350,12 +353,13 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 /* NB: don't set inkey here */
 #endif
                 if (debug)
-                    fprintf(stderr,"keyword defintion: \"%s\" on line %d.\n", line2 + 1, line_count);
+                    fprintf(stderr,"keyword definition: \"%s\" on line %d.\n", line2 + 1, line_count);
             }
 	    break;
     case '@':{			/* start/end table */
-            skiptable = !skiptable;
-            if (!skiptable) forcetable = FALSE;
+	    skiptable = !skiptable;
+	    if (!skiptable) forcetable = FALSE;
+	    intable = !intable; /* just for latex list support */
 	    break;
 	}
     case '^':{			/* html link escape */
@@ -364,11 +368,14 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 inhlink = TRUE;
                 /* remove trailing newline etc */
                 str = line + strlen(line) - 1;
-                while (*str=='\r' || *str=='\n') *str-- = NUL;
+                while (*str == '\r' || *str == '\n') *str-- = NUL;
                 fprintf(b, "%s", line + 2);
             } else if (inhlink) {
                 inhlink = FALSE;
 	        fputs(line + 2, b);	/* copy directly */
+	    } else if (strncmp(line, "^figure_", 8) == 0) {
+		/* fall through to embedded figure as in doc2web */
+		;
             } else {
                 if (line[2] == '!') { /* hack for function sections */
                     const char magic[] = "<!-- INCLUDE_NEXT_TABLE -->";
@@ -378,15 +385,40 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 inhlink = FALSE;
 	        fputs(line + 1, b);	/* copy directly */
             }
-	    break;		/* ignore */
+	    if (strncmp(line, "^figure_", 8) == 0)
+		; /* Fall through to embedded figure */
+	    else
+		break;
 	}
+    case 'D':			/* link to online demo figure */
+	    break;
     case 'F':			/* latex embedded figure */
             if (para) fprintf(b, "</p><p align=\"justify\">\n");
             fprintf(b, "<img src=\"%s.png\" alt=\"%s\">\n", line2+1, line2+1);
             if (para) fprintf(b, "</p><p align=\"justify\">\n");
             break;
     case '#':{			/* latex table entry */
-	    break;		/* ignore */
+	    if (!intable) {  /* HACK: we just handle itemized lists */
+		/* Itemized list outside of table */
+		if (line[1] == 's')
+		    (void) fputs("<ul>\n", b);
+		else if (line[1] == 'e')
+		    (void) fputs("</ul>\n", b);
+		else if (line[1] == 'b') {
+		    /* Bullet */
+		    fprintf(b, "<li>%s\n", line2+2);
+		}
+		else if (line[1] == '#') {
+		    /* Continuation of bulleted line */
+		    fputs(line2+2, b);
+		}
+		else if (strncmp(line+1, "TeX", 3) != 0) {
+		    if (strchr(line, '\n'))
+			*(strchr(line, '\n')) = '\0';
+		    fprintf(b, "<li><pre>%s</pre>\n", line + 1);
+		}
+	    }
+	    break;
 	}
     case '%':{			/* troff table entry */
 	    break;		/* ignore */
@@ -469,7 +501,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                     strcpy(location, name);
 
 #ifndef SPLIT_FILES
-                /* let hhc auomatically split contents */
+                /* let hhc automatically split contents */
                 if (!startpage) {
                     fprintf(b, "<OBJECT type=\"application/x-oleobject\" classid=\"clsid:1e2a7bd0-dab9-11d0-b93a-00c04fc99f9e\">\n");
 	            fprintf(b, "<param name=\"New HTML file\" value=\"%s.html\">\n", location);
@@ -480,15 +512,16 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
 #else
 		/* split contents manually */
                 if (!startpage) {
-		    char newfile[PATH_MAX];
+		    char newfile[PATH_MAX] = "";
 
                     /* close current file */
 		    footer(b);
 		    fclose(b);
 
                     /* open new file */
-                    sprintf(newfile, "%s%s.html", path, location);
-                    /* fprintf(stderr, "%s\n", newfile); */
+		    strcat(newfile,path);
+		    strncat(newfile,location,PATH_MAX-strlen(newfile)-6);
+		    strcat(newfile,".html");
                     if (!(b = fopen(newfile, "w"))) {
                         fprintf(stderr, "%s: Can't open %s for writing\n",
                             "doc2html", newfile);
