@@ -40,6 +40,7 @@
 #include "gp_time.h"
 #include "term_api.h"
 #include "gplocale.h"
+#include "mouse.h"	/* for inside_zoom() */
 
 /* HBB 20000725: gather all per-axis variables into a struct, and set
  * up a single large array of such structs. Next step might be to use
@@ -161,13 +162,11 @@ TBOOLEAN raxis = FALSE;
 double theta_origin = 0.0;	/* default origin at right side */
 double theta_direction = 1;	/* counterclockwise from origin */
 
-/* Length of the longest tics label, set by widest_tic_callback(): */
+/* Length of the longest tics label, set by widest_tic_callback() */
 int widest_tic_strlen;
 
-/* flag to indicate that in-line axis ranges should be ignored
- * and zoom/pan range limits take precedence over auto-scaling
- */
-TBOOLEAN inside_zoom;
+/* Number of axis tics actually placed, set by tic_count_callback() */
+static int axis_tic_count;
 
 /* axes being used by the current plot */
 /* These are mainly convenience variables, replacing separate copies of
@@ -271,6 +270,9 @@ axis_name(AXIS_INDEX axis)
 void
 init_sample_range(AXIS *axis, enum PLOT_TYPE plot_type)
 {
+    /* Special case: x2 linked to logscale x1 */
+    if (axis->forced_log_link > 0)
+	axis -= (SECOND_X_AXIS - FIRST_X_AXIS);
     axis_array[SAMPLE_AXIS].range_flags = 0;
     axis_array[SAMPLE_AXIS].min = axis->min;
     axis_array[SAMPLE_AXIS].max = axis->max;
@@ -426,7 +428,7 @@ axis_checked_extend_empty_range(AXIS_INDEX axis, const char *mesg)
 
     if (dmax - dmin == 0.0) {
 	/* empty range */
-	if (this_axis->autoscale || inside_zoom) {
+	if (this_axis->autoscale || inside_zoom()) {
 	    /* range came from autoscaling ==> widen it */
 	    double widen = (dmax == 0.0) ?
 		FIXUP_RANGE__WIDEN_ZERO_ABS
@@ -435,9 +437,9 @@ axis_checked_extend_empty_range(AXIS_INDEX axis, const char *mesg)
 		fprintf(stderr, "Warning: empty %s range [%g:%g], ",
 		    axis_name(axis), dmin, dmax);
 	    /* HBB 20010525: correctly handle single-ended autoscaling */
-	    if ((this_axis->autoscale & AUTOSCALE_MIN) || inside_zoom)
+	    if ((this_axis->autoscale & AUTOSCALE_MIN) || inside_zoom())
 		this_axis->min -= widen;
-	    if ((this_axis->autoscale & AUTOSCALE_MAX) || inside_zoom)
+	    if ((this_axis->autoscale & AUTOSCALE_MAX) || inside_zoom())
 		this_axis->max += widen;
 	    if (!(axis == FIRST_Z_AXIS && !mesg)) /* set view map */
 		fprintf(stderr, "adjusting to [%g:%g]\n",
@@ -607,7 +609,7 @@ copy_or_invent_formatstring(struct axis *this_axis)
 	/* The simple case: formatstring is usable, so use it! */
 	strncpy(tempfmt, this_axis->formatstring, MAX_ID_LEN);
 	/* Ensure enough precision to distinguish tics */
-	if (!strcmp(tempfmt, DEF_FORMAT)) {
+	if (is_def_format(tempfmt)) {
 	    double axmin = this_axis->min;
 	    double axmax = this_axis->max;
 	    int precision = ceil(-log10(GPMIN(fabs(axmax-axmin),fabs(axmin))));
@@ -686,10 +688,6 @@ copy_or_invent_formatstring(struct axis *this_axis)
  * hardwired into the calls to here. Maybe we will restore it
  * to the automatic calculation one day
  */
-
-/* HBB 20020220: Changed to use value itself as first argument, not
- * log10(value).  Done to allow changing the calculation method
- * to avoid numerical problems */
 double
 quantize_normal_tics(double arg, int guide)
 {
@@ -756,7 +754,8 @@ make_tics(struct axis *this_axis, int guide)
     }
 
     tic = quantize_normal_tics(xr, guide);
-    if (this_axis->log && tic < 1.0)
+    if ((this_axis->log && tic < 1.0)
+    &&  this_axis->ticdef.logscaling && !this_axis->ticdef.force_linear_tics)
 	tic = 1.0;
     if (this_axis->tictype == DT_TIMEDATE)
 	tic = quantize_time_tics(this_axis, tic, xr, guide);
@@ -931,10 +930,12 @@ setup_tics(struct axis *this, int max)
 	}
     }
 
-    /* HBB 20000506: if no tics required for this axis, do
-     * nothing. This used to be done exactly before each call of
-     * setup_tics, anyway... */
-    if (! this->ticmode)
+    /* If no tics required for this axis, do nothing */
+    if (!this->ticmode)
+	return;
+
+    /* Special case: x2 linked to logscale x1 */
+    if (this->forced_log_link > 0)
 	return;
 
     if (ticdef->type == TIC_SERIES) {
@@ -951,7 +952,7 @@ setup_tics(struct axis *this, int max)
     }
 
     /* It is disconcerting when the response to pan or zoom is asymmetric */
-    if (inside_zoom)
+    if (inside_zoom())
 	autoextend_min = autoextend_max = FALSE;
 
     /* If an explicit stepsize was set, axis->timelevel wasn't defined,
@@ -1132,6 +1133,16 @@ gen_tics(struct axis *this, tic_callback callback)
 	switch (def->type) {
 	case TIC_SERIES:
 	    if (this->log && this->index != POLAR_AXIS) {
+		struct axis *primary = this;
+		if (this->linked_to_primary)
+		    primary = this->linked_to_primary;
+		/* Special case: x2 linked to logscale x1 */
+		if (this->forced_log_link > 0) {
+		    this -= (SECOND_X_AXIS - FIRST_X_AXIS);
+		    primary = this->linked_to_primary;
+		    if (!primary)
+			int_error(NO_CARET, "log axis with broken link");
+		}
 		/* we can tolerate start <= 0 if step and end > 0 */
 		if (def->def.series.end <= 0 || def->def.series.incr <= 0)
 		    return;	/* just quietly ignore */
@@ -1144,21 +1155,21 @@ gen_tics(struct axis *this, tic_callback callback)
 		    end = step * ceil(lmax / step);
 		else
 		    end = def->def.series.end;
-		if (def->logscaling) {
+		if (def->logscaling && !def->force_linear_tics) {
 		    /* This tries to emulate earlier gnuplot versions in handling
 		     *     set log y; set ytics 10
 		     */
 		    if (start <= 0) {
 			start = step;
-			while (start > this->linked_to_primary->min)
+			while (start > primary->min)
 			    start -= step;
 		    } else {
-			start = eval_link_function(this->linked_to_primary, start);
+			start = eval_link_function(primary, start);
 		    }
-		    step  = eval_link_function(this->linked_to_primary, step);
-		    end   = eval_link_function(this->linked_to_primary, end);
-		    lmin = this->linked_to_primary->min;
-		    lmax = this->linked_to_primary->max;
+		    step  = eval_link_function(primary, step);
+		    end   = eval_link_function(primary, end);
+		    lmin = primary->min;
+		    lmax = primary->max;
 		}
 	    } else {
 		start = def->def.series.start;
@@ -1174,7 +1185,14 @@ gen_tics(struct axis *this, tic_callback callback)
 	    }
 	    break;
 	case TIC_COMPUTED:
-	    if (nonlinear(this)) {
+	    /* Special case: x2 linked to logscale x1 */
+	    if (this->forced_log_link > 0)
+		this -= (SECOND_X_AXIS - FIRST_X_AXIS);
+	    if (this->log
+	    && (!this->ticdef.logscaling || this->ticdef.force_linear_tics)) {
+		FPRINTF((stderr,"special case (log %s axis but linear tic intervals)\n",
+			axis_name(this->index)));
+	    } else if (nonlinear(this)) {
 		lmin = this->linked_to_primary->min;
 		lmax = this->linked_to_primary->max;
 		reorder_if_necessary(lmin, lmax);
@@ -1212,8 +1230,8 @@ gen_tics(struct axis *this, tic_callback callback)
 	step = fabs(step);
 
 	if (minitics != MINI_OFF) {
-	    FPRINTF((stderr,"axis.c: %d  start = %g end = %g step = %g base = %g\n",
-			__LINE__, start, end, step, this->base));
+	    FPRINTF((stderr,"axis.c: %d  %s start = %g end = %g step = %g base = %g\n",
+			__LINE__, axis_name(this->index), start, end, step, this->base));
 
 	    /* {{{  figure out ministart, ministep, miniend */
 	    if (minitics == MINI_USER) {
@@ -1231,7 +1249,7 @@ gen_tics(struct axis *this, tic_callback callback)
 		    ministart = ministep = step / this->mtic_freq;
 		    miniend = step;
  		}
-	    } else if (nonlinear(this) && this->ticdef.logscaling) {
+	    } else if (nonlinear(this) && this->ticdef.logscaling && !this->ticdef.force_linear_tics) {
 		ministart = ministep = step / (this->base - 1);
 		miniend = step;
 	    } else if (this->tictype == DT_TIMEDATE) {
@@ -1277,7 +1295,11 @@ gen_tics(struct axis *this, tic_callback callback)
 	/* }}} */
 
 	/* This protects against user error, not precision errors */
-	if ( (internal_max-internal_min)/step > term->xmax) {
+	/* Note: tick-counting via tic_count_callback can execute before the
+	 * terminal has been initialised.
+	 */
+	if (term_initialised
+	&&  (internal_max-internal_min)/step > term->xmax) {
 	    int_warn(NO_CARET,"Too many axis ticks requested (>%.0g)",
 		(internal_max-internal_min)/step);
 	    return;
@@ -1307,6 +1329,12 @@ gen_tics(struct axis *this, tic_callback callback)
 		nsteps--;
 	}
 
+	/* For debugging logscale tic generation */
+	FPRINTF((stderr, "gen_tics: %s axis %s logscaling %d force_linear %d\n",
+	    axis_name(this->index),
+	    def->type == 1 ? "COMPUTED" : def->type == 2 ? "SERIES" : "USER",
+	    def->logscaling, def->force_linear_tics));
+
 	for (tic = start; nsteps > 0; tic += step, nsteps--) {
 
 	    /* {{{  calc internal and user co-ords */
@@ -1315,8 +1343,12 @@ gen_tics(struct axis *this, tic_callback callback)
 		internal = tic;
 		user = tic;
 	    } else if (nonlinear(this)) {
-		if (def->type == TIC_SERIES && def->logscaling)
+		if (def->type == TIC_SERIES && def->logscaling && !def->force_linear_tics)
 		    user = eval_link_function(this, tic);
+		else if (def->type == TIC_SERIES)
+		    user = tic;
+		else if (def->type == TIC_COMPUTED && def->force_linear_tics)
+		    user = tic;
 		else if (def->type == TIC_COMPUTED)
 		    user = eval_link_function(this, tic);
 		else
@@ -1382,7 +1414,8 @@ gen_tics(struct axis *this, tic_callback callback)
 			}
 
 			/* This is where we finally decided to put the tic mark */
-			if (nonlinear(this) && (def->type == TIC_SERIES && def->logscaling))
+			if (nonlinear(this)
+			&& (def->type == TIC_SERIES && def->logscaling && !def->force_linear_tics))
 			    position = user;
 			else if (nonlinear(this) && (def->type == TIC_COMPUTED))
 			    position = user;
@@ -1414,21 +1447,57 @@ gen_tics(struct axis *this, tic_callback callback)
 		continue;
 
 	    if (minitics != MINI_OFF) {
-		/* {{{  process minitics */
+		/* process minitics */
 		double mplace, mtic_user, mtic_internal;
 
+		/* Make up for bad calculation of ministart/ministep/miniend.
+		 * Logscale with step > 1 means that the span between major tics
+		 * is more than one log unit.  In this case we position minitics
+		 * only at the otherwise unmarked unit log intervals.
+		 */
+		if (this->log && def->logscaling && !def->force_linear_tics) {
+		    if (step > 1) {
+			ministart = internal;
+			ministep = 1;
+			miniend = internal + step;
+			/* But don't go crazy with the minitics */
+			if ((miniend - ministart) > 10)
+			    continue;
+		    }
+		}
+
+		/* If the axis is logscaled but the major tics have been changed
+		 * from log to linear placement, adjust the minor tics to match.
+		 */
+		if (nonlinear(this) && this->log
+		&&  (!def->logscaling || (def->logscaling && def->force_linear_tics))) {
+		    ministep = step / 10.;
+		    ministart = internal + ministep;
+		    miniend = internal + step;
+		}
+
 		for (mplace = ministart; mplace < miniend; mplace += ministep) {
 		    if (this->tictype == DT_TIMEDATE) {
 			mtic_user = time_tic_just(this->timelevel - 1, internal + mplace);
 			mtic_internal = mtic_user;
-		    } else if ((nonlinear(this) && (def->type == TIC_COMPUTED))
-			   ||  (nonlinear(this) && (def->type == TIC_SERIES && def->logscaling))) {
+		    } else if ((nonlinear(this) && def->logscaling && !def->force_linear_tics)
+			   &&  ((def->type == TIC_COMPUTED) || (def->type == TIC_SERIES))) {
 			/* Make up for bad calculation of ministart/ministep/miniend */
-			double this_major = eval_link_function(this, internal);
-			double next_major = eval_link_function(this, internal+step);
-			mtic_user = this_major + mplace/miniend * (next_major - this_major);
-			mtic_internal = eval_link_function(this->linked_to_primary, mtic_user);
+			if (step > 1) {
+			    mtic_user = eval_link_function(this,mplace);
+			    mtic_internal = mplace;
+			} else {
+			    double this_major = eval_link_function(this, internal);
+			    double next_major = eval_link_function(this, internal+step);
+			    mtic_user = this_major + mplace/miniend * (next_major - this_major);
+			    mtic_internal = eval_link_function(this->linked_to_primary, mtic_user);
+			}
+		    } else if (nonlinear(this) && this->log
+			   &&  (!def->logscaling || (def->logscaling && def->force_linear_tics))) {
+			mtic_user = mplace;
+			mtic_internal = mtic_user;	/* It isn't really but this makes the range checks work */
 		    } else if (nonlinear(this) && this->log) {
+			/* FIXME - not sure this is correct. Fall through instead? */
 			mtic_user = internal + mplace;
 			mtic_internal = eval_link_function(this->linked_to_primary, mtic_user);
 		    } else {
@@ -1453,7 +1522,6 @@ gen_tics(struct axis *this, tic_callback callback)
 		    &&  inrange(mtic_internal, start - step * SIGNIF, end + step * SIGNIF))
 			(*callback) (this, mtic_user, NULL, 1, mgrd, NULL);
 		}
-		/* }}} */
 	    }
 	}
 
@@ -2012,6 +2080,38 @@ widest_tic_callback(struct axis *this_axis, double place, char *text,
     }
 }
 
+/* Sanity check for auto-generated tics on logscale axes.
+ * If fewer than 3 tics were generated we will switch to using
+ * linear increment tic intervals when they are drawn by place_grid().
+ */
+void
+sanity_check_log_tics( int axis_index )
+{
+    struct axis *axis = &axis_array[axis_index];
+    if (axis->ticmode == NO_TICS)
+	return;
+    if (axis->log) {
+	axis->ticdef.force_linear_tics = FALSE;
+	axis_tic_count = 0;			/* reset counter for actual number of tics */
+	gen_tics(axis, tic_count_callback);	/* find actual number of tics */
+	if (axis_tic_count < 3) {
+	    axis->ticdef.force_linear_tics = TRUE;
+	    FPRINTF((stderr, " >>>>>>>> forcing switch to linear tic generation along %s\n", axis_name(axis_index)));
+	    setup_tics(axis, 20);
+	}
+    }
+}
+
+/* this callback increments axis_tic_count for each tic placed */
+void
+tic_count_callback(struct axis *this_axis, double place, char *text,
+    int ticlevel, struct lp_style_type grid, struct ticmark *userlabels)
+{
+    if (ticlevel != 0)
+	return;
+    if (inrange(place, this_axis->min, this_axis->max))
+	axis_tic_count++;
+}
 
 /*
  * get and set routines for range writeback
@@ -2211,6 +2311,9 @@ add_tic_user(struct axis *this_axis, char *label, double position, int level)
     struct ticmark *tic, *newtic;
     struct ticmark listhead;
 
+    if (isnan(position))
+	return;
+
     if (!label && level < 0)
 	return;
 
@@ -2375,15 +2478,18 @@ char *c, *cfmt;
 /* Accepts a range of the form [MIN:MAX] or [var=MIN:MAX]
  * Loads new limiting values into axis->min axis->max
  * Returns
+ *	>0 = token indexing the leading variable name [foo=min:max]
  *	 0 = no range spec present
- *	-1 = range spec with no attached variable name
- *	>0 = token indexing the attached variable name
+ *	-1 = range spec with no leading variable name
+ *	-2 = found a second colon, so it must be a sampling range
  */
 int
 parse_range(AXIS_INDEX axis)
 {
     struct axis *this_axis = &axis_array[axis];
     int dummy_token = -1;
+    int starting_token = c_token;
+    struct axis saved_axis;
 
     /* No range present */
     if (!equals(c_token, "["))
@@ -2402,9 +2508,23 @@ parse_range(AXIS_INDEX axis)
 	c_token += 2;
     }
 
+    /* Save entire axis state before loading range, so we can back out if necessary */
+    memcpy(&saved_axis, this_axis, sizeof(struct axis));
+
     this_axis->autoscale = load_range(this_axis, &this_axis->min, &this_axis->max,
 					this_axis->autoscale);
 
+    /* A second colon is permitted only in a sampling range.
+     * If we see one now, back out from trying to interpret this as a primary axis.
+     */
+    if (equals(c_token, ":")
+    && !(axis == SAMPLE_AXIS || axis == T_AXIS || axis == U_AXIS || axis == V_AXIS)) {
+	c_token = starting_token;
+	/* restore original axis range */
+	memcpy(this_axis, &saved_axis, sizeof(struct axis));
+	return -2;
+    }
+
     /* Nonlinear axis - find the linear range equivalent */
     if (this_axis->linked_to_primary) {
 	AXIS *primary = this_axis->linked_to_primary;
@@ -2437,14 +2557,26 @@ parse_range(AXIS_INDEX axis)
     return dummy_token;
 }
 
-/* Called if an in-line range is encountered while inside a zoom command */
-void
+/* Called if an in-line range is encountered while inside a zoom command.
+ * On entry c_token indexes the opening '['.
+ */
+TBOOLEAN
 parse_skip_range()
 {
-    while (!equals(c_token++,"]"))
+    int start_token = c_token;
+    int ncolons = 0;
+
+    while (!equals(c_token++,"]")) {
+	if (equals(c_token,":"))
+	    ncolons++;
 	if (END_OF_COMMAND)
 	    break;
-    return;
+    }
+    if (ncolons >= 2) {
+	c_token = start_token;
+	return FALSE;
+    }
+    return TRUE;
 }
 
 /*
@@ -2457,7 +2589,14 @@ clone_linked_axes(AXIS *axis1, AXIS *axis2)
     double testmin, testmax, scale;
     TBOOLEAN suspect = FALSE;
 
+    double save_data_min = axis2->data_min;
+    double save_data_max = axis2->data_max;
+
     memcpy(axis2, axis1, AXIS_CLONE_SIZE);
+
+    axis2->data_min = save_data_min;
+    axis2->data_max = save_data_max;
+
     if (axis2->link_udf == NULL || axis2->link_udf->at == NULL)
 	return;
 
@@ -2614,10 +2753,15 @@ get_shadow_axis(AXIS *axis)
 void
 extend_primary_ticrange(AXIS *axis)
 {
-    AXIS *primary = axis->linked_to_primary;
-    TBOOLEAN autoextend_min = (axis->autoscale & AUTOSCALE_MIN)
+    AXIS *primary;
+    TBOOLEAN autoextend_min, autoextend_max;
+   
+    primary = axis->linked_to_primary;
+    if (!primary)	/* Special case: x2 linked to logscale x1 */
+	return;
+    autoextend_min = (axis->autoscale & AUTOSCALE_MIN)
 	&& !(axis->autoscale & AUTOSCALE_FIXMIN);
-    TBOOLEAN autoextend_max = (axis->autoscale & AUTOSCALE_MAX)
+    autoextend_max = (axis->autoscale & AUTOSCALE_MAX)
 	&& !(axis->autoscale & AUTOSCALE_FIXMAX);
 
     if (axis->ticdef.logscaling) {
@@ -2653,8 +2797,25 @@ update_primary_axis_range(struct axis *secondary)
 	/* nonlinear axis (secondary is visible; primary is hidden) */
 	primary->min = eval_link_function(primary, secondary->min);
 	primary->max = eval_link_function(primary, secondary->max);
-	primary->data_min = eval_link_function(primary, secondary->data_min);
-	primary->data_max = eval_link_function(primary, secondary->data_max);
+    }
+
+    /* Special case: x2 linked to logscale x1 */
+    if (secondary->forced_log_link < 0) {
+	/* This is merging the user-visible limits from x1 into x2 */
+	struct axis *forced_link = secondary + (SECOND_X_AXIS - FIRST_X_AXIS);
+	if (forced_link->min > secondary->min)
+	    forced_link->min = secondary->min;
+	if (forced_link->max < secondary->max)
+	    forced_link->max = secondary->max;
+    }
+    if (secondary->forced_log_link > 0) {
+	/* This is merging the user-visible limits from x2 into x1 */ 
+	struct axis *forced_link = secondary - (SECOND_X_AXIS - FIRST_X_AXIS);
+	if (forced_link->min > secondary->min)
+	    forced_link->min = secondary->min;
+	if (forced_link->max < secondary->max)
+	    forced_link->max = secondary->max;
+	update_primary_axis_range(forced_link);
     }
 }
 
@@ -2670,8 +2831,6 @@ update_secondary_axis_range(struct axis *primary)
     if (secondary != NULL) {
        secondary->min = eval_link_function(secondary, primary->min);
        secondary->max = eval_link_function(secondary, primary->max);
-       secondary->data_min = eval_link_function(secondary, primary->data_min);
-       secondary->data_max = eval_link_function(secondary, primary->data_max);
     }
 }
 
@@ -2690,32 +2849,30 @@ extend_autoscaled_log_axis(AXIS *primary)
 }
 
 /*
- * We track the x1 and x2 axis data limits separately.
- * However if x1 and x2 are linked to each other we must reconcile
- * their data limits before plotting.
+ * We track the x1/x2 and y1/y2 axis data limits separately.
+ * However if either pair is linked ("set link x2") both data ranges
+ * contribute to autoscaling the full plot.
  */
 void
 reconcile_linked_axes(AXIS *primary, AXIS *secondary)
 {
-    double dummy;
-    coord_type inrange = INRANGE;
     if ((primary->autoscale & AUTOSCALE_BOTH) != AUTOSCALE_NONE
     &&  primary->linked_to_secondary) {
-	double min_2_into_1 = eval_link_function(primary, secondary->data_min);
-	double max_2_into_1 = eval_link_function(primary, secondary->data_max);
-
-	/* Merge secondary min/max into primary data range */
-	store_and_update_range(&dummy, min_2_into_1, &inrange, primary, FALSE);
-	store_and_update_range(&dummy, max_2_into_1, &inrange, primary, FALSE);
-	(void)dummy;	/* Otherwise the compiler complains about an unused variable */
-
-	/* Take the result back the other way to update secondary */
-	secondary->min = eval_link_function(secondary, primary->min);
-	secondary->max = eval_link_function(secondary, primary->max);
+	double min_2 = eval_link_function(primary, secondary->data_min);
+	double max_2 = eval_link_function(primary, secondary->data_max);
+	double min_1 = eval_link_function(secondary, primary->data_min);
+	double max_1 = eval_link_function(secondary, primary->data_max);
+	if (primary->min > min_2)
+	    primary->min = min_2;
+	if (primary->max < max_2)
+	    primary->max = max_2;
+	if (secondary->min > min_1)
+	    secondary->min = min_1;
+	if (secondary->max < max_1)
+	    secondary->max = max_1;
     }
 }
 
-
 /*
  * Check for linked-axis coordinate transformation given by command
  *     set {x|y}2r link via <expr1> inverse <expr2>
@@ -2726,8 +2883,14 @@ reconcile_linked_axes(AXIS *primary, AXIS *secondary)
 double
 map_x_double(double value)
 {
-    if (axis_array[x_axis].linked_to_primary) {
-	AXIS *primary = axis_array[x_axis].linked_to_primary;
+    AXIS *xaxis = &axis_array[x_axis];
+
+    /* Special case: x2 linked to logscale x1 */
+    if (xaxis->forced_log_link > 0)
+	xaxis = &axis_array[FIRST_X_AXIS];
+
+    if (xaxis->linked_to_primary) {
+	AXIS *primary = xaxis->linked_to_primary;
 	if (primary->link_udf->at) {
 	    value = eval_link_function(primary, value);
 	    if (undefined)
@@ -2750,8 +2913,14 @@ map_x(double value)
 double
 map_y_double(double value)
 {
-    if (axis_array[y_axis].linked_to_primary) {
-	AXIS *primary = axis_array[y_axis].linked_to_primary;
+    AXIS *yaxis = &axis_array[y_axis];
+
+    /* Special case: y2 linked to logscale y1 */
+    if (yaxis->forced_log_link > 0)
+	yaxis = &axis_array[FIRST_Y_AXIS];
+
+    if (yaxis->linked_to_primary) {
+	AXIS *primary = yaxis->linked_to_primary;
 	if (primary->link_udf->at) {
 	    value = eval_link_function(primary, value);
 	    if (undefined)
